import { BlockModelInterface, CardModelInterface, ClipboardInterface, CommandInterface, ConversionInterface, EditorInterface, EditorOptions, EventInterface, EventListener, InlineModelInterface, LanguageInterface, MarkModelInterface, NodeIdInterface, NodeInterface, NodeModelInterface, PluginModelInterface, RangeInterface, RequestInterface, SchemaInterface, Selector } from './types';
import { ListModelInterface } from './types/list';
declare class Editor<T extends EditorOptions = EditorOptions> implements EditorInterface<T> {
    #private;
    readonly kind: 'editor' | 'engine' | 'view';
    options: T;
    readonly container: NodeInterface;
    language: LanguageInterface;
    root: NodeInterface;
    card: CardModelInterface;
    plugin: PluginModelInterface;
    node: NodeModelInterface;
    nodeId: NodeIdInterface;
    list: ListModelInterface;
    mark: MarkModelInterface;
    inline: InlineModelInterface;
    block: BlockModelInterface;
    event: EventInterface;
    schema: SchemaInterface;
    conversion: ConversionInterface;
    command: CommandInterface;
    clipboard: ClipboardInterface;
    request: RequestInterface;
    get scrollNode(): NodeInterface | null;
    constructor(selector: Selector, options?: EditorOptions);
    init(): void;
    setScrollNode(node?: HTMLElement): void;
    on<R = any, F extends EventListener<R> = EventListener<R>>(eventType: string, listener: F, rewrite?: boolean): this;
    off(eventType: string, listener: EventListener): this;
    trigger<R = any>(eventType: string, ...args: any): R;
    messageSuccess(message: string): void;
    messageError(error: string): void;
    messageConfirm(message: string): Promise<boolean>;
    getSelectionData(range?: RangeInterface): Record<'html' | 'text', string> | undefined;
    destroy(): void;
}
export default Editor;
