/// <reference types="node" />
import { EventEmitter2 } from 'eventemitter2';
import { EngineInterface } from '../types/engine';
import { Op, Path, Doc } from 'sharedb';
import { NodeInterface } from '../types/node';
import { DocInterface, RepairOp } from '../types/ot';
declare class Producer extends EventEmitter2 {
    private engine;
    private doc?;
    private dmp;
    private cacheNodes;
    private cacheTransientElements?;
    timer: NodeJS.Timeout | null;
    lineStart: boolean;
    constructor(engine: EngineInterface, options: {
        doc?: DocInterface | Doc;
    });
    textToOps(path: Path, text1: string, text2: string): (import("sharedb/lib/sharedb").StringDeleteOp | import("sharedb/lib/sharedb").StringInsertOp)[];
    cacheNode(node: Node): void;
    clearCache(): void;
    inCache(node: Node): Node | undefined;
    isTransientMutation(record: MutationRecord, transientElements?: Array<Node>, loadingCards?: NodeInterface[]): boolean;
    filter: (element: NodeInterface | Node) => ((node: Node) => boolean) | undefined;
    getPath: (root: NodeInterface, node: NodeInterface) => number[];
    private indexCaches;
    getIndex: (element: NodeInterface) => number;
    /**
     * 从DOM变更记录中生产 ops
     * @param records DOM变更记录集合
     * @param path 路径
     * @param oldPath
     * @param node 开始遍历的节点，默认为编辑器根节点
     * @returns
     */
    generateOps(records: MutationRecord[], node?: NodeInterface): Array<Op>;
    /**
     * 从 doc 中查找目标卡片
     * @param data
     * @param name
     * @param callback
     * @returns 返回卡片属性，以及是否已渲染
     */
    findCardForDoc: (data: any, name: string, callback?: ((attriables: {
        [key: string]: string;
    }) => boolean) | undefined) => {
        attriables: any;
        rendered: boolean;
    } | void;
    /**
     * 处理DOM节点变更记录
     * @param records 记录集合
     */
    handleMutations(records: MutationRecord[]): void;
    emitOps(ops: ((RepairOp & {
        newValue?: string;
        path?: number[];
    }) | Op)[]): void;
    setDoc(doc: DocInterface | Doc): void;
    /**
     * 获取要移除节点的索引
     * @param record 当前记录
     * @param records 记录集合
     * @returns
     */
    getRemoveNodeIndex(record: MutationRecord, records: MutationRecord[]): number;
    getRemoveNodeIndexFromMutation(node: Node, target: Node, records: MutationRecord[]): number;
}
export default Producer;
